<?php

require_once 'exception/ApiValidationException.php';
use EasyCredit\Process\Process;
use EasyCredit\Transfer\Customer;
use EasyCredit\Transfer\PersonData;
use EasyCredit\Api\ApiClient;
use EasyCredit\Api\DataMapper;
use EasyCredit\Http\Request;
use EasyCredit\Http\Adapter\Curl;
use EasyCredit\Log\Logger;
use EasyCredit\Transfer\RiskRelatedInfo;
use EasyCredit\Transfer\ProcessData;
use EasyCredit\Transfer\ProcessInitialize;
use EasyCredit\Transfer\TechnicalShopParams;
use EasyCredit\Transfer\DeliveryAddress;
use EasyCredit\Transfer\Contact;
use EasyCredit\Transfer\BillingAddress;
use EasyCredit\Transfer\ArticleIdCollection;
use EasyCredit\Transfer\ArticleId;
use EasyCredit\Transfer\CartInfo;
use EasyCredit\Transfer\CartInfoCollection;
use EasyCredit\Transfer\CallbackUrls;
use EasyCredit\Transfer\ProcessSaveInput;

/**
 * Class EasyCreditApiClient
 */
class EasyCreditApiClient
{
    const EASYCREDIT_API_HOSTNAME = 'ratenkauf.easycredit.de';
    const EASYCREDIT_API_PORT = 443;

    /**
     * @var string
     */
    protected $webShopId;

    /**
     * @var string
     */
    protected $webShopToken;

    /**
     * 
     * @var Process
     */
    protected $ecProcess;
    
    /**
     * @var \EasyCredit\Api\ApiClient
     */
    protected $apiClient;
    
    

    /**
     * EasyCreditApiClient constructor.
     * @param string $webShopId
     * @param string $webShopToken
     */
    public function __construct($webShopId, $webShopToken)
    {
        $this->webShopId    = $webShopId;
        $this->webShopToken = $webShopToken;
        $logger             = new Logger(new GambioHandler(), strtolower(MODULE_PAYMENT_EASYCREDIT_LOGGING));
        $requestAdapter     = new Curl();
        $requestAdapter->setLogger($logger);

        if (trim(MODULE_PAYMENT_EASYCREDIT_PROXY_HOST) != "" && trim(MODULE_PAYMENT_EASYCREDIT_PROXY_PORT) != "") {
            $requestAdapter->setProxyEnabled(true);
            $requestAdapter->setProxyHost(MODULE_PAYMENT_EASYCREDIT_PROXY_HOST);
            $requestAdapter->setProxyPort(MODULE_PAYMENT_EASYCREDIT_PROXY_PORT);

            if (trim(MODULE_PAYMENT_EASYCREDIT_PROXY_USERNAME) != ""
                && trim(MODULE_PAYMENT_EASYCREDIT_PROXY_PASSWORD) != ""
            ) {
                $requestAdapter->setProxyUsername(MODULE_PAYMENT_EASYCREDIT_PROXY_USERNAME);
                $requestAdapter->setProxyPassword(MODULE_PAYMENT_EASYCREDIT_PROXY_PASSWORD);
            }
        }

        $request = new Request(
            self::EASYCREDIT_API_HOSTNAME, self::EASYCREDIT_API_PORT, $requestAdapter
        );
        $dataMapper = new DataMapper();

        $this->apiClient = new ApiClient(
            $this->webShopId,
            $this->webShopToken,
            $request,
            $dataMapper
        );
        
        try {
            $this->ecProcess = Process::createInstance($this->apiClient);
        } catch (Exception $e) {
            $this->ecProcess = Process::getInstance();
        }
    }

    /**
     * @param float $amount
     * @return array
     */
    public function getModelCalculation($amount)
    {
        $modelCalculcation = $this->apiClient->get($amount);

        return $modelCalculcation->toArray();
    }

    /**
     * @param float $amount
     * @return array
     */
    public function getBestModelCalculation($amount)
    {
        $modelCalculcation = $this->apiClient->getBest($amount);

        return $modelCalculcation->toArray();
    }

    /**
     * @param order $order
     * @param array $products
     * @param float $orderTotal
     */
    public function initialize(order $order, array $products, $orderTotal)
    {
        if ($order->customer['gender'] == "m") {
            $salutation = PersonData::SALUTATION_MR;
        } else {
            $salutation = PersonData::SALUTATION_MRS;
        }
        
        $birthDate  = $this->getBirthDate((int)$_SESSION['customer_id']);
        $personData = new PersonData();
        
        if($this->is_date($birthDate)) {
            $personData->setBirthDate($birthDate);
        }
        $personData->setFirstName($order->customer['firstname']);
        $personData->setLastName($order->customer['lastname']);
        $personData->setSalutation($salutation);
        
        $customer = new Customer();
        $customer->setPersonData($personData);
        $customer->setContact($this->getContact($order));

        $riskRelatedInfo = new RiskRelatedInfo();
        $riskRelatedInfo->setNegativePaymentInformation(
            \EasyCredit\Transfer\RiskRelatedInfo::NEGATIVE_PAYMENT_INFORMATION_NO_INFORMATION
        );
        $riskRelatedInfo->setOrderCount($_SESSION['easycredit_data']['order_count']);

        $customerRegistratedAt = $this->getRegisteredDate((int)$_SESSION['customer_id']);
        if (intval($_SESSION['customers_status']['customers_status_id']) == 1 || !$this->is_date($customerRegistratedAt)) {
            $riskRelatedInfo->setCustomerRegistrated(false);
        } else {
            $riskRelatedInfo->setCustomerRegistrated(true);
            $datet = new DateTime( trim($customerRegistratedAt) );
            $riskRelatedInfo->setCustomerRegistrationDate($datet->format('Y-m-d'));
        }
        $riskRelatedInfo->setRiskItemInCart(false);

        $cartInfos = new CartInfoCollection();

        foreach ($products as $product) {
            $cartInfo = new CartInfo();
            $cartInfo->setName($product['name']);
            $cartInfo->setQuantity($product['qty']);
            $cartInfo->setPrice($product['price']);
            if (!empty($product['category'])) {
                $cartInfo->setCategory($product['category']);
            }
            if (!empty($product['ean'])) {
                $articleIdCollection = new ArticleIdCollection();

                $articleId = new ArticleId();
                $articleId->setId($product['ean']);
                $articleId->setType(\EasyCredit\Transfer\ArticleId::TYPE_EAN);

                $articleIdCollection->addItem($articleId);
                $cartInfo->setArticleId($articleIdCollection);
            }
            $cartInfos->addItem($cartInfo);

            $completeQty += $product['qty'];
        }
        $riskRelatedInfo->setCartItemsCount($completeQty);
        
        $processData = $this->getEcProcess()->getProcessData();
        $processData->setCustomer($customer);
        $processData->setBillingAddress($this->getBillingAddress($order));
        $processData->setDeliveryAddress($this->getDeliveryAddress($order));
        $processData->setOrderTotal((float)$orderTotal);
        $processData->setRiskInfo($riskRelatedInfo);
        $processData->setProducts($cartInfos);
        
        $callbackUrls = new CallbackUrls();
        $callbackUrls->setUrlSucceeded(xtc_href_link('checkout_confirmation.php?ecretval=1', '', 'SSL'));
        $callbackUrls->setUrlCancelled(xtc_href_link('checkout_payment.php', '', 'SSL'));
        $callbackUrls->setUrlDenied(xtc_href_link('checkout_confirmation.php?ecretval=denied', '', 'SSL'));
        $processData->setCallbackUrls($callbackUrls);
        
        $processData->setTerm((int)$_SESSION['easycredit_data']['number_of_rates']);
        
        
        $technicalShopParams = new TechnicalShopParams();
        $technicalShopParams->setShopPlatformManufacturer('Gambio ' . gm_get_conf('INSTALLED_VERSION'));
        $technicalShopParams->setShopPlatformModuleVersion('Version 1.0.0');

        $processInitialized = $this->ecProcess->initialize(ProcessInitialize::INTEGRATION_TYPE_PAYMENT_PAGE, $technicalShopParams);

        if (!$processInitialized) {
            $messages = $this->ecProcess->getMessages();
        }

        if (isset($messages['messages'])) {
            $this->setGlobalApiMessages($messages);
        }

        $_SESSION['easycredit_data']['tb_process_identifier'] = $this->ecProcess->getProcessData()->getTbaId();
        $_SESSION['easycredit_data']['technical_process_identifier'] = $this->ecProcess->getProcessData()->getTechnicalTbaId();
    }

    /**
     * @return \EasyCredit\Transfer\BillingAddress
     */
    public function getBillingAddress($order)
    {
        $billingAddress = new BillingAddress();
        if (array_key_exists('house_number', $order->billing) && !empty($order->billing['house_number'])) {
            $billingAddress->setStreet($order->billing['street_address'] . ' ' . $order->billing['house_number']);
        } else {
            $billingAddress->setStreet($order->billing['street_address']);
        }
        $billingAddress->setCity($order->billing['city']);
        $billingAddress->setZip($order->billing['postcode']);
        $billingAddress->setCountryCode($order->billing['country']['iso_code_2']);

        return $billingAddress;
    }

    /**
     * @return \EasyCredit\Transfer\DeliveryAddress
     */
    public function getDeliveryAddress($order)
    {
        if (!isset($order->delivery) || !is_array($order->delivery)) {
            return null;
        }

        $deliveryAddress = new DeliveryAddress();
        if (array_key_exists('house_number', $order->delivery) && !empty($order->delivery['house_number'])) {
            $deliveryAddress->setStreet($order->delivery['street_address'] . ' ' . $order->delivery['house_number']);
        } else {
            $deliveryAddress->setStreet($order->delivery['street_address']);
        }
        $deliveryAddress->setCity($order->delivery['city']);
        $deliveryAddress->setZip($order->delivery['postcode']);
        $deliveryAddress->setCountryCode($order->delivery['country']['iso_code_2']);
        $deliveryAddress->setFirstName($order->customer['firstname']);
        $deliveryAddress->setLastName($order->customer['lastname']);

        return $deliveryAddress;
    }

    /**
     * @return \EasyCredit\Transfer\Contact
     */
    public function getContact($order)
    {
        $contact = new Contact();
        $contact->setEmail($order->customer['email_address']);
        $contact->setMobilphone($_SESSION['easycredit_data']['mobile_phone']);

        if (isset($_SESSION['easycredit_data']['mobile_phone_verify'])
            && (int)$_SESSION['easycredit_data']['mobile_phone_verify'] == 1
        ) {
            $contact->setMobilphoneVerify(true);
        }

        return $contact;
    }

    /**
     * @param order $order
     * @param array $products
     *
     * @return \EasyCredit\Transfer\GetDecisionResponse|boolean
     */
    public function process(order $order)
    {
//         return $this->apiClient->decide($_SESSION['easycredit_data']['tb_process_identifier']);
        return $this->apiClient->getDecision($_SESSION['easycredit_data']['tb_process_identifier']);
    }

    /**
     * @param array $messages
     */
    public function setGlobalApiMessages($messages)
    {
        $apiMessages = array();
        foreach ($messages['messages'] as $message) {
            if (isset($message['renderedMessage']) && !empty($message['renderedMessage'])) {
                $apiMessages[strtolower($message['severity'])][$message['key']] = $message['renderedMessage'];
            }
        }

        $_SESSION['easycredit_api_messages'] = $apiMessages;
    }

    /**
     * @param string $tbProcessIdentifier
     * @return \EasyCredit\Transfer\BaseResponse
     */
    public function agreeInstallment($tbProcessIdentifier)
    {
        return $this->apiClient->agreeInstallment($tbProcessIdentifier);
    }

    /**
     * @param string $tbProcessIdentifier
     * @return \EasyCredit\Transfer\TransferInterface
     */
    public function getFinancingDetails($tbProcessIdentifier)
    {
        return $this->apiClient->getFinancingDetails($tbProcessIdentifier);
    }

    /**
     * @return \EasyCredit\Transfer\LegislativeText
     */
    public function getLegislativeText()
    {
        return $this->apiClient->getLegislativeText();
    }

    /**
     * @param string $tbProcessIdentifier
     * @return \EasyCredit\Transfer\GetPersonResponse
     */
    public function getProcessPersonDetail($tbProcessIdentifier)
    {
        return $this->apiClient->getProcessPersonDetail($tbProcessIdentifier);
    }

    /**
     *
     * @return Process
     */
    public function getEcProcess()
    {
        return $this->ecProcess;
    }

    /**
     *
     * @param Process $ecProcess            
     */
    public function setEcProcess(Process $ecProcess)
    {
        $this->ecProcess = $ecProcess;
    }
 
    protected function getBirthDate($customerId) {
        $customer_query = xtc_db_query("select customers_dob from customers where customers_id = '". $customerId ."'");
        $c_vals = xtc_db_fetch_array($customer_query);
        
        return $c_vals['customers_dob'];
    }
    
    protected function getRegisteredDate($customerId) {
        $customer_query = xtc_db_query("select customers_date_added from customers where customers_id = '". $customerId ."'");
        $c_vals = xtc_db_fetch_array($customer_query);
    
        return $c_vals['customers_date_added'];
    }

    /**
     * @param string $str
     *
     * @return boolean
     */
    protected function is_date( $str ) {
        if($str == '1000-01-01 00:00:00') {
            return false;
        }
        try {
            $dt = new DateTime( trim($str) );
        }
        catch( Exception $e ) {
            return false;
        }
    
        $month = $dt->format('m');
        $day   = $dt->format('d');
        $year  = $dt->format('Y');
    
        if( checkdate($month, $day, $year) ) {
            return true;
        }
        else {
            return false;
        }
    }
    
    /**
     * 
     * @param order $order
     * @return boolean
     */
    public function isEnabledCountry(order $order) {
        return strtoupper($order->billing['country']['iso_code_2']) == 'DE' ? true : false;
    }
    
    /**
     * 
     * @param string $tbProcessIdentifier
     * @return \EasyCredit\Transfer\CommonProcessData
     */
    public function getCommonProcessData($tbProcessIdentifier) {
        return $this->apiClient->getCommonProcessData($tbProcessIdentifier);
    }
}
